/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.alterinfo.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.alterinfo.entity.AlterInfo;
import com.jeeplus.modules.alterinfo.service.AlterInfoService;

/**
 * 变更作废明细Controller
 * @author lw
 * @version 2018-04-13
 */
@Controller
@RequestMapping(value = "${adminPath}/alterinfo/alterInfo")
public class AlterInfoController extends BaseController {

	@Autowired
	private AlterInfoService alterInfoService;
	
	@ModelAttribute
	public AlterInfo get(@RequestParam(required=false) String id) {
		AlterInfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = alterInfoService.get(id);
		}
		if (entity == null){
			entity = new AlterInfo();
		}
		return entity;
	}
	
	/**
	 * 变更作废明细列表页面
	 */
	@RequiresPermissions("alterinfo:alterInfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(AlterInfo alterInfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<AlterInfo> page = alterInfoService.findPage(new Page<AlterInfo>(request, response), alterInfo); 
		model.addAttribute("page", page);
		return "modules/alterinfo/alterInfoList";
	}

	/**
	 * 查看，增加，编辑变更作废明细表单页面
	 */
	@RequiresPermissions(value={"alterinfo:alterInfo:view","alterinfo:alterInfo:add","alterinfo:alterInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(AlterInfo alterInfo, Model model) {
		model.addAttribute("alterInfo", alterInfo);
		return "modules/alterinfo/alterInfoForm";
	}

	/**
	 * 保存变更作废明细
	 */
	@RequiresPermissions(value={"alterinfo:alterInfo:add","alterinfo:alterInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(AlterInfo alterInfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, alterInfo)){
			return form(alterInfo, model);
		}
		if(!alterInfo.getIsNewRecord()){//编辑表单保存
			AlterInfo t = alterInfoService.get(alterInfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(alterInfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			alterInfoService.save(t);//保存
		}else{//新增表单保存
			alterInfoService.save(alterInfo);//保存
		}
		addMessage(redirectAttributes, "保存变更作废明细成功");
		return "redirect:"+Global.getAdminPath()+"/alterinfo/alterInfo/?repage";
	}
	
	/**
	 * 删除变更作废明细
	 */
	@RequiresPermissions("alterinfo:alterInfo:del")
	@RequestMapping(value = "delete")
	public String delete(AlterInfo alterInfo, RedirectAttributes redirectAttributes) {
		alterInfoService.delete(alterInfo);
		addMessage(redirectAttributes, "删除变更作废明细成功");
		return "redirect:"+Global.getAdminPath()+"/alterinfo/alterInfo/?repage";
	}
	
	/**
	 * 批量删除变更作废明细
	 */
	@RequiresPermissions("alterinfo:alterInfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			alterInfoService.delete(alterInfoService.get(id));
		}
		addMessage(redirectAttributes, "删除变更作废明细成功");
		return "redirect:"+Global.getAdminPath()+"/alterinfo/alterInfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("alterinfo:alterInfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(AlterInfo alterInfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "变更作废明细"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<AlterInfo> page = alterInfoService.findPage(new Page<AlterInfo>(request, response, -1), alterInfo);
    		new ExportExcel("变更作废明细", AlterInfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出变更作废明细记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/alterinfo/alterInfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("alterinfo:alterInfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<AlterInfo> list = ei.getDataList(AlterInfo.class);
			for (AlterInfo alterInfo : list){
				try{
					alterInfoService.save(alterInfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条变更作废明细记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条变更作废明细记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入变更作废明细失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/alterinfo/alterInfo/?repage";
    }
	
	/**
	 * 下载导入变更作废明细数据模板
	 */
	@RequiresPermissions("alterinfo:alterInfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "变更作废明细数据导入模板.xlsx";
    		List<AlterInfo> list = Lists.newArrayList(); 
    		new ExportExcel("变更作废明细数据", AlterInfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/alterinfo/alterInfo/?repage";
    }
	
	
	

}