/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.casebuildinfo.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.casebuildinfo.entity.CaseBuildInfo;
import com.jeeplus.modules.casebuildinfo.service.CaseBuildInfoService;

/**
 * 案例-建设单位Controller
 * @author fgy
 * @version 2018-01-24
 */
@Controller
@RequestMapping(value = "${adminPath}/casebuildinfo/caseBuildInfo")
public class CaseBuildInfoController extends BaseController {

	@Autowired
	private CaseBuildInfoService caseBuildInfoService;
	
	@ModelAttribute
	public CaseBuildInfo get(@RequestParam(required=false) String id) {
		CaseBuildInfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = caseBuildInfoService.get(id);
		}
		if (entity == null){
			entity = new CaseBuildInfo();
		}
		return entity;
	}
	
	/**
	 * 案例-建设单位列表页面
	 */
	@RequiresPermissions("casebuildinfo:caseBuildInfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(CaseBuildInfo caseBuildInfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<CaseBuildInfo> page = caseBuildInfoService.findPage(new Page<CaseBuildInfo>(request, response), caseBuildInfo); 
		model.addAttribute("page", page);
		return "modules/casebuildinfo/caseBuildInfoList";
	}

	/**
	 * 查看，增加，编辑案例-建设单位表单页面
	 */
	@RequiresPermissions(value={"casebuildinfo:caseBuildInfo:view","casebuildinfo:caseBuildInfo:add","casebuildinfo:caseBuildInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(CaseBuildInfo caseBuildInfo, Model model) {
		model.addAttribute("caseBuildInfo", caseBuildInfo);
		return "modules/casebuildinfo/caseBuildInfoForm";
	}

	/**
	 * 保存案例-建设单位
	 */
	@RequiresPermissions(value={"casebuildinfo:caseBuildInfo:add","casebuildinfo:caseBuildInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(CaseBuildInfo caseBuildInfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, caseBuildInfo)){
			return form(caseBuildInfo, model);
		}
		if(!caseBuildInfo.getIsNewRecord()){//编辑表单保存
			CaseBuildInfo t = caseBuildInfoService.get(caseBuildInfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(caseBuildInfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			caseBuildInfoService.save(t);//保存
		}else{//新增表单保存
			caseBuildInfoService.save(caseBuildInfo);//保存
		}
		addMessage(redirectAttributes, "保存案例-建设单位成功");
		return "redirect:"+Global.getAdminPath()+"/casebuildinfo/caseBuildInfo/?repage";
	}
	
	/**
	 * 删除案例-建设单位
	 */
	@RequiresPermissions("casebuildinfo:caseBuildInfo:del")
	@RequestMapping(value = "delete")
	public String delete(CaseBuildInfo caseBuildInfo, RedirectAttributes redirectAttributes) {
		caseBuildInfoService.delete(caseBuildInfo);
		addMessage(redirectAttributes, "删除案例-建设单位成功");
		return "redirect:"+Global.getAdminPath()+"/casebuildinfo/caseBuildInfo/?repage";
	}
	
	/**
	 * 批量删除案例-建设单位
	 */
	@RequiresPermissions("casebuildinfo:caseBuildInfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			caseBuildInfoService.delete(caseBuildInfoService.get(id));
		}
		addMessage(redirectAttributes, "删除案例-建设单位成功");
		return "redirect:"+Global.getAdminPath()+"/casebuildinfo/caseBuildInfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("casebuildinfo:caseBuildInfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(CaseBuildInfo caseBuildInfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-建设单位"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<CaseBuildInfo> page = caseBuildInfoService.findPage(new Page<CaseBuildInfo>(request, response, -1), caseBuildInfo);
    		new ExportExcel("案例-建设单位", CaseBuildInfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出案例-建设单位记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casebuildinfo/caseBuildInfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("casebuildinfo:caseBuildInfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<CaseBuildInfo> list = ei.getDataList(CaseBuildInfo.class);
			for (CaseBuildInfo caseBuildInfo : list){
				try{
					caseBuildInfoService.save(caseBuildInfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条案例-建设单位记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条案例-建设单位记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入案例-建设单位失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casebuildinfo/caseBuildInfo/?repage";
    }
	
	/**
	 * 下载导入案例-建设单位数据模板
	 */
	@RequiresPermissions("casebuildinfo:caseBuildInfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-建设单位数据导入模板.xlsx";
    		List<CaseBuildInfo> list = Lists.newArrayList(); 
    		new ExportExcel("案例-建设单位数据", CaseBuildInfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casebuildinfo/caseBuildInfo/?repage";
    }
	
	
	

}