/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.casemechanicalnorm.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.casemechanicalnorm.entity.CaseMechanicalNorm;
import com.jeeplus.modules.casemechanicalnorm.service.CaseMechanicalNormService;

/**
 * 机械消耗量指标Controller
 * @author liuw
 * @version 2018-02-07
 */
@Controller
@RequestMapping(value = "${adminPath}/casemechanicalnorm/caseMechanicalNorm")
public class CaseMechanicalNormController extends BaseController {

	@Autowired
	private CaseMechanicalNormService caseMechanicalNormService;
	
	@ModelAttribute
	public CaseMechanicalNorm get(@RequestParam(required=false) String id) {
		CaseMechanicalNorm entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = caseMechanicalNormService.get(id);
		}
		if (entity == null){
			entity = new CaseMechanicalNorm();
		}
		return entity;
	}
	
	/**
	 * 机械消耗量指标列表页面
	 */
	@RequiresPermissions("casemechanicalnorm:caseMechanicalNorm:list")
	@RequestMapping(value = {"list", ""})
	public String list(CaseMechanicalNorm caseMechanicalNorm, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<CaseMechanicalNorm> page = caseMechanicalNormService.findPage(new Page<CaseMechanicalNorm>(request, response), caseMechanicalNorm); 
		model.addAttribute("page", page);
		return "modules/casemechanicalnorm/caseMechanicalNormList";
	}

	/**
	 * 查看，增加，编辑机械消耗量指标表单页面
	 */
	@RequiresPermissions(value={"casemechanicalnorm:caseMechanicalNorm:view","casemechanicalnorm:caseMechanicalNorm:add","casemechanicalnorm:caseMechanicalNorm:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(CaseMechanicalNorm caseMechanicalNorm, Model model) {
		model.addAttribute("caseMechanicalNorm", caseMechanicalNorm);
		return "modules/casemechanicalnorm/caseMechanicalNormForm";
	}

	/**
	 * 保存机械消耗量指标
	 */
	@RequiresPermissions(value={"casemechanicalnorm:caseMechanicalNorm:add","casemechanicalnorm:caseMechanicalNorm:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(CaseMechanicalNorm caseMechanicalNorm, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, caseMechanicalNorm)){
			return form(caseMechanicalNorm, model);
		}
		if(!caseMechanicalNorm.getIsNewRecord()){//编辑表单保存
			CaseMechanicalNorm t = caseMechanicalNormService.get(caseMechanicalNorm.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(caseMechanicalNorm, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			caseMechanicalNormService.save(t);//保存
		}else{//新增表单保存
			caseMechanicalNormService.save(caseMechanicalNorm);//保存
		}
		addMessage(redirectAttributes, "保存机械消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/casemechanicalnorm/caseMechanicalNorm/?repage";
	}
	
	/**
	 * 删除机械消耗量指标
	 */
	@RequiresPermissions("casemechanicalnorm:caseMechanicalNorm:del")
	@RequestMapping(value = "delete")
	public String delete(CaseMechanicalNorm caseMechanicalNorm, RedirectAttributes redirectAttributes) {
		caseMechanicalNormService.delete(caseMechanicalNorm);
		addMessage(redirectAttributes, "删除机械消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/casemechanicalnorm/caseMechanicalNorm/?repage";
	}
	
	/**
	 * 批量删除机械消耗量指标
	 */
	@RequiresPermissions("casemechanicalnorm:caseMechanicalNorm:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			caseMechanicalNormService.delete(caseMechanicalNormService.get(id));
		}
		addMessage(redirectAttributes, "删除机械消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/casemechanicalnorm/caseMechanicalNorm/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("casemechanicalnorm:caseMechanicalNorm:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(CaseMechanicalNorm caseMechanicalNorm, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "机械消耗量指标"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<CaseMechanicalNorm> page = caseMechanicalNormService.findPage(new Page<CaseMechanicalNorm>(request, response, -1), caseMechanicalNorm);
    		new ExportExcel("机械消耗量指标", CaseMechanicalNorm.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出机械消耗量指标记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casemechanicalnorm/caseMechanicalNorm/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("casemechanicalnorm:caseMechanicalNorm:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<CaseMechanicalNorm> list = ei.getDataList(CaseMechanicalNorm.class);
			for (CaseMechanicalNorm caseMechanicalNorm : list){
				try{
					caseMechanicalNormService.save(caseMechanicalNorm);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条机械消耗量指标记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条机械消耗量指标记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入机械消耗量指标失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casemechanicalnorm/caseMechanicalNorm/?repage";
    }
	
	/**
	 * 下载导入机械消耗量指标数据模板
	 */
	@RequiresPermissions("casemechanicalnorm:caseMechanicalNorm:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "机械消耗量指标数据导入模板.xlsx";
    		List<CaseMechanicalNorm> list = Lists.newArrayList(); 
    		new ExportExcel("机械消耗量指标数据", CaseMechanicalNorm.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casemechanicalnorm/caseMechanicalNorm/?repage";
    }
	
	
	

}