/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.casepeoplenorm.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.casepeoplenorm.entity.CasePeopleNorm;
import com.jeeplus.modules.casepeoplenorm.service.CasePeopleNormService;

/**
 * 人工消耗量指标Controller
 * @author liuw
 * @version 2018-02-07
 */
@Controller
@RequestMapping(value = "${adminPath}/casepeoplenorm/casePeopleNorm")
public class CasePeopleNormController extends BaseController {

	@Autowired
	private CasePeopleNormService casePeopleNormService;
	
	@ModelAttribute
	public CasePeopleNorm get(@RequestParam(required=false) String id) {
		CasePeopleNorm entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = casePeopleNormService.get(id);
		}
		if (entity == null){
			entity = new CasePeopleNorm();
		}
		return entity;
	}
	
	/**
	 * 人工消耗量指标列表页面
	 */
	@RequiresPermissions("casepeoplenorm:casePeopleNorm:list")
	@RequestMapping(value = {"list", ""})
	public String list(CasePeopleNorm casePeopleNorm, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<CasePeopleNorm> page = casePeopleNormService.findPage(new Page<CasePeopleNorm>(request, response), casePeopleNorm); 
		model.addAttribute("page", page);
		return "modules/casepeoplenorm/casePeopleNormList";
	}

	/**
	 * 查看，增加，编辑人工消耗量指标表单页面
	 */
	@RequiresPermissions(value={"casepeoplenorm:casePeopleNorm:view","casepeoplenorm:casePeopleNorm:add","casepeoplenorm:casePeopleNorm:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(CasePeopleNorm casePeopleNorm, Model model) {
		model.addAttribute("casePeopleNorm", casePeopleNorm);
		return "modules/casepeoplenorm/casePeopleNormForm";
	}

	/**
	 * 保存人工消耗量指标
	 */
	@RequiresPermissions(value={"casepeoplenorm:casePeopleNorm:add","casepeoplenorm:casePeopleNorm:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(CasePeopleNorm casePeopleNorm, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, casePeopleNorm)){
			return form(casePeopleNorm, model);
		}
		if(!casePeopleNorm.getIsNewRecord()){//编辑表单保存
			CasePeopleNorm t = casePeopleNormService.get(casePeopleNorm.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(casePeopleNorm, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			casePeopleNormService.save(t);//保存
		}else{//新增表单保存
			casePeopleNormService.save(casePeopleNorm);//保存
		}
		addMessage(redirectAttributes, "保存人工消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/casepeoplenorm/casePeopleNorm/?repage";
	}
	
	/**
	 * 删除人工消耗量指标
	 */
	@RequiresPermissions("casepeoplenorm:casePeopleNorm:del")
	@RequestMapping(value = "delete")
	public String delete(CasePeopleNorm casePeopleNorm, RedirectAttributes redirectAttributes) {
		casePeopleNormService.delete(casePeopleNorm);
		addMessage(redirectAttributes, "删除人工消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/casepeoplenorm/casePeopleNorm/?repage";
	}
	
	/**
	 * 批量删除人工消耗量指标
	 */
	@RequiresPermissions("casepeoplenorm:casePeopleNorm:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			casePeopleNormService.delete(casePeopleNormService.get(id));
		}
		addMessage(redirectAttributes, "删除人工消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/casepeoplenorm/casePeopleNorm/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("casepeoplenorm:casePeopleNorm:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(CasePeopleNorm casePeopleNorm, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "人工消耗量指标"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<CasePeopleNorm> page = casePeopleNormService.findPage(new Page<CasePeopleNorm>(request, response, -1), casePeopleNorm);
    		new ExportExcel("人工消耗量指标", CasePeopleNorm.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出人工消耗量指标记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casepeoplenorm/casePeopleNorm/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("casepeoplenorm:casePeopleNorm:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<CasePeopleNorm> list = ei.getDataList(CasePeopleNorm.class);
			for (CasePeopleNorm casePeopleNorm : list){
				try{
					casePeopleNormService.save(casePeopleNorm);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条人工消耗量指标记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条人工消耗量指标记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入人工消耗量指标失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casepeoplenorm/casePeopleNorm/?repage";
    }
	
	/**
	 * 下载导入人工消耗量指标数据模板
	 */
	@RequiresPermissions("casepeoplenorm:casePeopleNorm:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "人工消耗量指标数据导入模板.xlsx";
    		List<CasePeopleNorm> list = Lists.newArrayList(); 
    		new ExportExcel("人工消耗量指标数据", CasePeopleNorm.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casepeoplenorm/casePeopleNorm/?repage";
    }
	
	
	

}