/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.exampleexpend.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.exampleexpend.entity.ExampleExpend;
import com.jeeplus.modules.exampleexpend.service.ExampleExpendService;

/**
 * 主要工料价格及消耗量指标Controller
 * @author ssrh
 * @version 2018-08-29
 */
@Controller
@RequestMapping(value = "${adminPath}/exampleexpend/exampleExpend")
public class ExampleExpendController extends BaseController {

	@Autowired
	private ExampleExpendService exampleExpendService;
	
	@ModelAttribute
	public ExampleExpend get(@RequestParam(required=false) String id) {
		ExampleExpend entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = exampleExpendService.get(id);
		}
		if (entity == null){
			entity = new ExampleExpend();
		}
		return entity;
	}
	
	/**
	 * 主要工料价格及消耗量指标列表页面
	 */
	@RequiresPermissions("exampleexpend:exampleExpend:list")
	@RequestMapping(value = {"list", ""})
	public String list(ExampleExpend exampleExpend, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ExampleExpend> page = exampleExpendService.findPage(new Page<ExampleExpend>(request, response), exampleExpend); 
		model.addAttribute("page", page);
		return "modules/exampleexpend/exampleExpendList";
	}

	/**
	 * 查看，增加，编辑主要工料价格及消耗量指标表单页面
	 */
	@RequiresPermissions(value={"exampleexpend:exampleExpend:view","exampleexpend:exampleExpend:add","exampleexpend:exampleExpend:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ExampleExpend exampleExpend, Model model) {
		model.addAttribute("exampleExpend", exampleExpend);
		return "modules/exampleexpend/exampleExpendForm";
	}

	/**
	 * 保存主要工料价格及消耗量指标
	 */
	@RequiresPermissions(value={"exampleexpend:exampleExpend:add","exampleexpend:exampleExpend:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ExampleExpend exampleExpend, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, exampleExpend)){
			return form(exampleExpend, model);
		}
		if(!exampleExpend.getIsNewRecord()){//编辑表单保存
			ExampleExpend t = exampleExpendService.get(exampleExpend.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(exampleExpend, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			exampleExpendService.save(t);//保存
		}else{//新增表单保存
			exampleExpendService.save(exampleExpend);//保存
		}
		addMessage(redirectAttributes, "保存主要工料价格及消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/exampleexpend/exampleExpend/?repage";
	}
	
	/**
	 * 删除主要工料价格及消耗量指标
	 */
	@RequiresPermissions("exampleexpend:exampleExpend:del")
	@RequestMapping(value = "delete")
	public String delete(ExampleExpend exampleExpend, RedirectAttributes redirectAttributes) {
		exampleExpendService.delete(exampleExpend);
		addMessage(redirectAttributes, "删除主要工料价格及消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/exampleexpend/exampleExpend/?repage";
	}
	
	/**
	 * 批量删除主要工料价格及消耗量指标
	 */
	@RequiresPermissions("exampleexpend:exampleExpend:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			exampleExpendService.delete(exampleExpendService.get(id));
		}
		addMessage(redirectAttributes, "删除主要工料价格及消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/exampleexpend/exampleExpend/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("exampleexpend:exampleExpend:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ExampleExpend exampleExpend, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "主要工料价格及消耗量指标"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ExampleExpend> page = exampleExpendService.findPage(new Page<ExampleExpend>(request, response, -1), exampleExpend);
    		new ExportExcel("主要工料价格及消耗量指标", ExampleExpend.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出主要工料价格及消耗量指标记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/exampleexpend/exampleExpend/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("exampleexpend:exampleExpend:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ExampleExpend> list = ei.getDataList(ExampleExpend.class);
			for (ExampleExpend exampleExpend : list){
				try{
					exampleExpendService.save(exampleExpend);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条主要工料价格及消耗量指标记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条主要工料价格及消耗量指标记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入主要工料价格及消耗量指标失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/exampleexpend/exampleExpend/?repage";
    }
	
	/**
	 * 下载导入主要工料价格及消耗量指标数据模板
	 */
	@RequiresPermissions("exampleexpend:exampleExpend:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "主要工料价格及消耗量指标数据导入模板.xlsx";
    		List<ExampleExpend> list = Lists.newArrayList(); 
    		new ExportExcel("主要工料价格及消耗量指标数据", ExampleExpend.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/exampleexpend/exampleExpend/?repage";
    }
	
	
	

}