/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.officelicense.web;

import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.entity.Office;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.officelicense.entity.OfficeLicense;
import com.jeeplus.modules.officelicense.service.OfficeLicenseService;

/**
 * 企业证照Controller
 * @author lw
 * @version 2018-08-14
 */
@Controller
@RequestMapping(value = "${adminPath}/officelicense/officeLicense")
public class OfficeLicenseController extends BaseController {

	@Autowired
	private OfficeLicenseService officeLicenseService;
	@Autowired
	private HttpServletRequest request;
	@ModelAttribute
	public OfficeLicense get(@RequestParam(required=false) String id) {
		OfficeLicense entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = officeLicenseService.get(id);
		}
		if (entity == null){
			entity = new OfficeLicense();
		}
		return entity;
	}
	
	/**
	 * 企业证照列表页面
	 */
	@RequiresPermissions("officelicense:officeLicense:list")
	@RequestMapping(value = {"list", ""})
	public String list(OfficeLicense officeLicense, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<OfficeLicense> page = officeLicenseService.findPage(new Page<OfficeLicense>(request, response), officeLicense); 
		model.addAttribute("page", page);
		return "modules/officelicense/officeLicenseList";
	}

	/**
	 * 查看，增加，编辑企业证照表单页面
	 */
	@RequiresPermissions(value={"officelicense:officeLicense:view","officelicense:officeLicense:add","officelicense:officeLicense:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(OfficeLicense officeLicense, Model model) {

		if(StringUtils.isBlank(officeLicense.getId())){
			Office office = UserUtils.getSelectOffice();
			officeLicense.setCompanyName(office.getParent().getName());
			officeLicense.setUploadDate(new Date());
		}
		model.addAttribute("officeLicense", officeLicense);
		String view=request.getParameter("tabId");
		if(StringUtils.isNotBlank(view) && view.equals("view")){
			return "modules/officelicense/officeLicenseView";
		}

		return "modules/officelicense/officeLicenseForm";
	}

	/**
	 * 保存企业证照
	 */
	@RequiresPermissions(value={"officelicense:officeLicense:add","officelicense:officeLicense:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(OfficeLicense officeLicense, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, officeLicense)){
			return form(officeLicense, model);
		}
		if(!officeLicense.getIsNewRecord()){//编辑表单保存
			OfficeLicense t = officeLicenseService.get(officeLicense.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(officeLicense, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			officeLicenseService.save(t);//保存
		}else{//新增表单保存
			officeLicense.setCompanyId(UserUtils.getSelectCompany().getId());
			officeLicenseService.save(officeLicense);//保存
		}
		addMessage(redirectAttributes, "保存企业证照成功");
		return "redirect:"+Global.getAdminPath()+"/officelicense/officeLicense/?repage";
	}
	
	/**
	 * 删除企业证照
	 */
	@RequiresPermissions("officelicense:officeLicense:del")
	@RequestMapping(value = "delete")
	public String delete(OfficeLicense officeLicense, RedirectAttributes redirectAttributes) {
		officeLicenseService.delete(officeLicense);
		addMessage(redirectAttributes, "删除企业证照成功");
		return "redirect:"+Global.getAdminPath()+"/officelicense/officeLicense/?repage";
	}
	
	/**
	 * 批量删除企业证照
	 */
	@RequiresPermissions("officelicense:officeLicense:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			officeLicenseService.delete(officeLicenseService.get(id));
		}
		addMessage(redirectAttributes, "删除企业证照成功");
		return "redirect:"+Global.getAdminPath()+"/officelicense/officeLicense/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("officelicense:officeLicense:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(OfficeLicense officeLicense, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "企业证照"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<OfficeLicense> page = officeLicenseService.findPage(new Page<OfficeLicense>(request, response, -1), officeLicense);
    		new ExportExcel("企业证照", OfficeLicense.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出企业证照记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officelicense/officeLicense/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("officelicense:officeLicense:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<OfficeLicense> list = ei.getDataList(OfficeLicense.class);
			for (OfficeLicense officeLicense : list){
				try{
					officeLicenseService.save(officeLicense);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条企业证照记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条企业证照记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入企业证照失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officelicense/officeLicense/?repage";
    }
	
	/**
	 * 下载导入企业证照数据模板
	 */
	@RequiresPermissions("officelicense:officeLicense:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "企业证照数据导入模板.xlsx";
    		List<OfficeLicense> list = Lists.newArrayList(); 
    		new ExportExcel("企业证照数据", OfficeLicense.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officelicense/officeLicense/?repage";
    }
	
	
	

}