/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.officequalify.web;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.jeeplus.modules.sys.entity.Office;
import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.officequalify.entity.Officequalify;
import com.jeeplus.modules.officequalify.service.OfficequalifyService;

/**
 * 企业资质Controller
 * @author liuw
 * @version 2017-12-05
 */
@Controller
@RequestMapping(value = "${adminPath}/officequalify/officequalify")
public class OfficequalifyController extends BaseController {

	@Autowired
	private OfficequalifyService officequalifyService;
	
	@ModelAttribute
	public Officequalify get(@RequestParam(required=false) String id) {
		Officequalify entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = officequalifyService.get(id);
		}
		if (entity == null){
			entity = new Officequalify();
		}
		return entity;
	}
	
	/**
	 * 企业资质列表页面
	 */
	@RequiresPermissions("officequalify:officequalify:list")
	@RequestMapping(value = {"list", ""})
	public String list(Officequalify officequalify, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Officequalify> page = officequalifyService.findPage(new Page<Officequalify>(request, response), officequalify);
		model.addAttribute("page", page);
		return "modules/officequalify/officequalifyList";
	}

	/**
	 * 查看，增加，编辑企业资质表单页面
	 */
	@RequiresPermissions(value={"officequalify:officequalify:view","officequalify:officequalify:add","officequalify:officequalify:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Officequalify officequalify, Model model,HttpServletRequest request) {
		if(StringUtils.isBlank(officequalify.getId())){
			Office office = UserUtils.getSelectOffice();
			officequalify.setCompanyName(office.getParent().getName());
			officequalify.setUploadDate(new Date());
		}
		model.addAttribute("officequalify", officequalify);
		String view=request.getParameter("tabId");
		if(StringUtils.isNotBlank(view) && view.equals("view")){
			return "modules/officequalify/officequalifyView";
		}
		return "modules/officequalify/officequalifyForm";
	}

	/**
	 * 保存企业资质
	 */
	@RequiresPermissions(value={"officequalify:officequalify:add","officequalify:officequalify:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Officequalify officequalify, Model model, RedirectAttributes redirectAttributes,HttpServletRequest request) throws Exception{
	//public String save(Officequalify officequalify, Model model, RedirectAttributes redirectAttributes,@RequestParam("next_upload_files") MultipartFile next_upload_files,HttpServletRequest request) throws Exception{
		if (!beanValidator(model, officequalify)){
			return form(officequalify, model,request);
		}
		if(!officequalify.getIsNewRecord()){//编辑表单保存
			Officequalify t = officequalifyService.get(officequalify.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(officequalify, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			officequalifyService.save(t);//保存
		}else{//新增表单保存
			officequalify.setCompanyId(UserUtils.getSelectCompany().getId());
			officequalifyService.save(officequalify);//保存
		}
		addMessage(redirectAttributes, "保存企业资质成功");
		return "redirect:"+Global.getAdminPath()+"/officequalify/officequalify/?repage";
	}
	
	/**
	 * 删除企业资质
	 */
	@RequiresPermissions("officequalify:officequalify:del")
	@RequestMapping(value = "delete")
	public String delete(Officequalify officequalify, RedirectAttributes redirectAttributes) {
		officequalifyService.delete(officequalify);
		addMessage(redirectAttributes, "删除企业资质成功");
		return "redirect:"+Global.getAdminPath()+"/officequalify/officequalify/?repage";
	}
	
	/**
	 * 批量删除企业资质
	 */
	@RequiresPermissions("officequalify:officequalify:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			officequalifyService.delete(officequalifyService.get(id));
		}
		addMessage(redirectAttributes, "删除企业资质成功");
		return "redirect:"+Global.getAdminPath()+"/officequalify/officequalify/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("officequalify:officequalify:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Officequalify officequalify, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "企业资质"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Officequalify> page = officequalifyService.findPage(new Page<Officequalify>(request, response, -1), officequalify);
    		new ExportExcel("企业资质", Officequalify.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出企业资质记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officequalify/officequalify/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("officequalify:officequalify:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Officequalify> list = ei.getDataList(Officequalify.class);
			for (Officequalify officequalify : list){
				try{
					officequalifyService.save(officequalify);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条企业资质记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条企业资质记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入企业资质失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officequalify/officequalify/?repage";
    }
	
	/**
	 * 下载导入企业资质数据模板
	 */
	@RequiresPermissions("officequalify:officequalify:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "企业资质数据导入模板.xlsx";
    		List<Officequalify> list = Lists.newArrayList(); 
    		new ExportExcel("企业资质数据", Officequalify.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officequalify/officequalify/?repage";
    }
	
	
	/**
	 * 选择企业外键
	 */
	@RequestMapping(value = "selectoffice")
	public String selectoffice(Office office, String url, String fieldLabels, String fieldKeys, String searchLabel, String searchKey, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Office> page = officequalifyService.findPageByoffice(new Page<Office>(request, response),  office);
		try {
			fieldLabels = URLDecoder.decode(fieldLabels, "UTF-8");
			fieldKeys = URLDecoder.decode(fieldKeys, "UTF-8");
			searchLabel = URLDecoder.decode(searchLabel, "UTF-8");
			searchKey = URLDecoder.decode(searchKey, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		model.addAttribute("labelNames", fieldLabels.split("\\|"));
		model.addAttribute("labelValues", fieldKeys.split("\\|"));
		model.addAttribute("fieldLabels", fieldLabels);
		model.addAttribute("fieldKeys", fieldKeys);
		model.addAttribute("url", url);
		model.addAttribute("searchLabel", searchLabel);
		model.addAttribute("searchKey", searchKey);
		model.addAttribute("obj", office);
		model.addAttribute("page", page);
		return "modules/sys/gridselect";
	}
	

}