/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.project.web;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.project.entity.Archivedirectory;
import com.jeeplus.modules.project.service.ArchivedirectoryService;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.project.entity.BidInformation;
import com.jeeplus.modules.project.service.BidInformationService;

/**
 * 标段信息Controller
 * @author 姚儒彬
 * @version 2017-10-11
 */
@Controller
@RequestMapping(value = "${adminPath}/project/bidInformation")
public class BidInformationController extends BaseController {

	@Autowired
	private BidInformationService bidInformationService;
	@Autowired
	private ArchivedirectoryService archivedirectoryService;


	@ModelAttribute
	public BidInformation get(@RequestParam(required=false) String id) {
		BidInformation entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = bidInformationService.get(id);
		}
		if (entity == null){
			entity = new BidInformation();
		}
		return entity;
	}
	
	/**
	 * 标段信息列表页面
	 */
	@RequiresPermissions("project:bidInformation:list")
	@RequestMapping(value = {"list", ""})
	public String list(BidInformation bidInformation, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<BidInformation> page = bidInformationService.findPage(new Page<BidInformation>(request, response), bidInformation); 
		System.out.print("page----------------------------page++"+page.getList());
		model.addAttribute("page", page);
		return "modules/project/bidInformationList";
	}

	/**
	 * 查看，增加，编辑标段信息表单页面
	 */
	@RequiresPermissions(value={"project:bidInformation:view","project:bidInformation:add","project:bidInformation:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(BidInformation bidInformation, Model model,String id) {
		bidInformation = bidInformationService.get(id);
		List<Archivedirectory> archivedirectoryList = archivedirectoryService.getbidInformationById(id);
		System.out.println("------------------------archivedirectoryList-----------"+archivedirectoryList);
		model.addAttribute("archivedirectoryList", archivedirectoryList);
		model.addAttribute("bidInformation", bidInformation);
		return "modules/project/bidInformationForm";
	}

	/**
	 * 查看标段信息表单页面
	 */
	@RequiresPermissions(value={"project:bidInformation:view"},logical=Logical.OR)
	@RequestMapping(value = "forms")
	public String forms(BidInformation bidInformation, Model model,String id) {
		bidInformation = bidInformationService.get(id);
		List<Archivedirectory> archivedirectoryList = archivedirectoryService.getbidInformationById(id);
		System.out.println("------------------------archivedirectoryList-----------"+archivedirectoryList);
		model.addAttribute("archivedirectoryList", archivedirectoryList);
		model.addAttribute("bidInformation", bidInformation);
		return "modules/project/bidInformationForms";
	}

	/**
	 * 保存标段信息
	 */
	@RequiresPermissions(value={"project:bidInformation:add","project:bidInformation:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(BidInformation bidInformation, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, bidInformation)){
			return form(bidInformation, model,"");
		}
		if(!bidInformation.getIsNewRecord()){//编辑表单保存
			BidInformation t = bidInformationService.get(bidInformation.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(bidInformation, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			bidInformationService.save(t);//保存
		}else{//新增表单保存
			bidInformationService.save(bidInformation);//保存
		}
		addMessage(redirectAttributes, "保存标段信息成功");
		return "redirect:"+Global.getAdminPath()+"/project/bidInformation/?repage";
	}
	
	/**
	 * 删除标段信息
	 */
	@RequiresPermissions("project:bidInformation:del")
	@RequestMapping(value = "delete")
	public String delete(BidInformation bidInformation, RedirectAttributes redirectAttributes) {
		bidInformationService.delete(bidInformation);
		addMessage(redirectAttributes, "删除标段信息成功");
		return "redirect:"+Global.getAdminPath()+"/project/bidInformation/?repage";
	}

	/**
	 * 批量删除标段信息
	 */
	@RequiresPermissions("project:bidInformation:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			bidInformationService.delete(bidInformationService.get(id));
		}
		addMessage(redirectAttributes, "删除标段信息成功");
		return "redirect:"+Global.getAdminPath()+"/project/bidInformation/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("project:bidInformation:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(BidInformation bidInformation, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "标段信息"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<BidInformation> page = bidInformationService.findPage(new Page<BidInformation>(request, response, -1), bidInformation);
    		new ExportExcel("标段信息", BidInformation.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出标段信息记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/bidInformation/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("project:bidInformation:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<BidInformation> list = ei.getDataList(BidInformation.class);
			for (BidInformation bidInformation : list){
				try{
					bidInformationService.save(bidInformation);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条标段信息记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条标段信息记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入标段信息失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/bidInformation/?repage";
    }
	
	/**
	 * 下载导入标段信息数据模板
	 */
	@RequiresPermissions("project:bidInformation:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "标段信息数据导入模板.xlsx";
    		List<BidInformation> list = Lists.newArrayList(); 
    		new ExportExcel("标段信息数据", BidInformation.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/bidInformation/?repage";
    }
	
	
	

}