/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.projectcontentinfo.web;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.common.json.AjaxJson;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.projectcontentinfo.entity.ProjectBasedData;
import com.jeeplus.modules.projectcontentinfo.service.ProjectBasedDataService;

/**
 * 依据性资料Controller
 * @author 杨帆
 * @version 2018-06-05
 */
@Controller
@RequestMapping(value = "${adminPath}/projectcontentinfo/projectBasedData")
public class ProjectBasedDataController extends BaseController {

	@Autowired
	private ProjectBasedDataService projectBasedDataService;
	
	@ModelAttribute
	public ProjectBasedData get(@RequestParam(required=false) String id) {
		ProjectBasedData entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = projectBasedDataService.get(id);
		}
		if (entity == null){
			entity = new ProjectBasedData();
		}
		return entity;
	}
	
	/**
	 * 依据性资料列表页面
	 */
	@RequiresPermissions("projectcontentinfo:projectBasedData:list")
	@RequestMapping(value = {"list", ""})
	public String list(ProjectBasedData projectBasedData, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ProjectBasedData> page = projectBasedDataService.findPage(new Page<ProjectBasedData>(request, response), projectBasedData); 
		model.addAttribute("page", page);
		return "modules/projectcontentinfo/projectBasedDataList";
	}

	/**
	 * 查看，增加，编辑依据性资料表单页面
	 */
	@RequiresPermissions(value={"projectcontentinfo:projectBasedData:view","projectcontentinfo:projectBasedData:add","projectcontentinfo:projectBasedData:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ProjectBasedData projectBasedData, Model model) {
		model.addAttribute("projectBasedData", projectBasedData);
		return "modules/projectcontentinfo/projectBasedDataForm";
	}

	/**
	 * 保存依据性资料
	 */
	@RequiresPermissions(value={"projectcontentinfo:projectBasedData:add","projectcontentinfo:projectBasedData:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ProjectBasedData projectBasedData, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, projectBasedData)){
			return form(projectBasedData, model);
		}
		if(!projectBasedData.getIsNewRecord()){//编辑表单保存
			ProjectBasedData t = projectBasedDataService.get(projectBasedData.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(projectBasedData, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			projectBasedDataService.save(t);//保存
		}else{//新增表单保存
			projectBasedDataService.save(projectBasedData);//保存
		}
		addMessage(redirectAttributes, "保存依据性资料成功");
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectBasedData/?repage";
	}
	
	/**
	 * 删除依据性资料
	 */
	@RequiresPermissions("projectcontentinfo:projectBasedData:del")
	@RequestMapping(value = "delete")
	public String delete(ProjectBasedData projectBasedData, RedirectAttributes redirectAttributes) {
		projectBasedDataService.delete(projectBasedData);
		addMessage(redirectAttributes, "删除依据性资料成功");
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectBasedData/?repage";
	}
	
	/**
	 * 批量删除依据性资料
	 */
	@RequiresPermissions("projectcontentinfo:projectBasedData:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			projectBasedDataService.delete(projectBasedDataService.get(id));
		}
		addMessage(redirectAttributes, "删除依据性资料成功");
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectBasedData/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("projectcontentinfo:projectBasedData:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ProjectBasedData projectBasedData, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "依据性资料"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ProjectBasedData> page = projectBasedDataService.findPage(new Page<ProjectBasedData>(request, response, -1), projectBasedData);
    		new ExportExcel("依据性资料", ProjectBasedData.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出依据性资料记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectBasedData/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("projectcontentinfo:projectBasedData:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ProjectBasedData> list = ei.getDataList(ProjectBasedData.class);
			for (ProjectBasedData projectBasedData : list){
				try{
					projectBasedDataService.save(projectBasedData);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条依据性资料记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条依据性资料记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入依据性资料失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectBasedData/?repage";
    }
	
	/**
	 * 下载导入依据性资料数据模板
	 */
	@RequiresPermissions("projectcontentinfo:projectBasedData:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "依据性资料数据导入模板.xlsx";
    		List<ProjectBasedData> list = Lists.newArrayList(); 
    		new ExportExcel("依据性资料数据", ProjectBasedData.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectBasedData/?repage";
    }


    @RequestMapping("selectList")
    public String gridSelect(ProjectBasedData projectBasedData,Model model,String searchLabel,HttpServletRequest request,HttpServletResponse response) throws UnsupportedEncodingException {
        Page<ProjectBasedData> page = projectBasedDataService.findPage(new Page<ProjectBasedData>(request,response), projectBasedData);
		String searchLabel1 = URLDecoder.decode(searchLabel,"utf-8");
        model.addAttribute("obj", projectBasedData);
        model.addAttribute("page", page);
        model.addAttribute("searchLabel",searchLabel1);
        return "modules/sys/gridselectProjbasedata";
    }

}