/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sysmodular.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.sysmodular.entity.SysModularField;
import com.jeeplus.modules.sysmodular.service.SysModularFieldService;

/**
 * 子段关系Controller
 * @author ssrh
 * @version 2018-08-17
 */
@Controller
@RequestMapping(value = "${adminPath}/sysmodular/sysModularField")
public class SysModularFieldController extends BaseController {

	@Autowired
	private SysModularFieldService sysModularFieldService;
	
	@ModelAttribute
	public SysModularField get(@RequestParam(required=false) String id) {
		SysModularField entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = sysModularFieldService.get(id);
		}
		if (entity == null){
			entity = new SysModularField();
		}
		return entity;
	}
	
	/**
	 * 子段关系列表页面
	 */
	//@RequiresPermissions("sysmodular:sysModularField:list")
	@RequestMapping(value = {"list", ""})
	public String list(SysModularField sysModularField, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<SysModularField> page = sysModularFieldService.findPage(new Page<SysModularField>(request, response), sysModularField); 
		model.addAttribute("page", page);
		return "modules/sysmodular/sysModularFieldList";
	}

	/**
	 * 查看，增加，编辑子段关系表单页面
	 */
	@RequiresPermissions(value={"sysmodular:sysModularField:view","sysmodular:sysModularField:add","sysmodular:sysModularField:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SysModularField sysModularField, Model model) {
		model.addAttribute("sysModularField", sysModularField);
		return "modules/sysmodular/sysModularFieldForm";
	}

	/**
	 * 保存子段关系
	 */
	@RequiresPermissions(value={"sysmodular:sysModularField:add","sysmodular:sysModularField:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SysModularField sysModularField, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, sysModularField)){
			return form(sysModularField, model);
		}
		if(!sysModularField.getIsNewRecord()){//编辑表单保存
			SysModularField t = sysModularFieldService.get(sysModularField.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(sysModularField, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			sysModularFieldService.save(t);//保存
		}else{//新增表单保存
			sysModularFieldService.save(sysModularField);//保存
		}
		addMessage(redirectAttributes, "保存子段关系成功");
		return "redirect:"+Global.getAdminPath()+"/sysmodular/sysModularField/?repage";
	}
	
	/**
	 * 删除子段关系
	 */
	@RequiresPermissions("sysmodular:sysModularField:del")
	@RequestMapping(value = "delete")
	public String delete(SysModularField sysModularField, RedirectAttributes redirectAttributes) {
		sysModularFieldService.delete(sysModularField);
		addMessage(redirectAttributes, "删除子段关系成功");
		return "redirect:"+Global.getAdminPath()+"/sysmodular/sysModularField/?repage";
	}
	
	/**
	 * 批量删除子段关系
	 */
	@RequiresPermissions("sysmodular:sysModularField:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			sysModularFieldService.delete(sysModularFieldService.get(id));
		}
		addMessage(redirectAttributes, "删除子段关系成功");
		return "redirect:"+Global.getAdminPath()+"/sysmodular/sysModularField/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("sysmodular:sysModularField:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SysModularField sysModularField, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "子段关系"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SysModularField> page = sysModularFieldService.findPage(new Page<SysModularField>(request, response, -1), sysModularField);
    		new ExportExcel("子段关系", SysModularField.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出子段关系记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysmodular/sysModularField/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("sysmodular:sysModularField:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SysModularField> list = ei.getDataList(SysModularField.class);
			for (SysModularField sysModularField : list){
				try{
					sysModularFieldService.save(sysModularField);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条子段关系记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条子段关系记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入子段关系失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysmodular/sysModularField/?repage";
    }
	
	/**
	 * 下载导入子段关系数据模板
	 */
	@RequiresPermissions("sysmodular:sysModularField:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "子段关系数据导入模板.xlsx";
    		List<SysModularField> list = Lists.newArrayList(); 
    		new ExportExcel("子段关系数据", SysModularField.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysmodular/sysModularField/?repage";
    }
	
	
	

}