/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcommunicaterecord.web;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workcommunicaterecord.entity.WorkCommunicateContent;
import com.jeeplus.modules.workcommunicaterecord.entity.WorkCommunicateRecord;
import com.jeeplus.modules.workcommunicaterecord.service.WorkCommunicateContentService;
import com.jeeplus.modules.workcommunicaterecord.service.WorkCommunicateRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 沟通记录Controller
 * @author 马鹏波
 * @version 2018-06-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workcommunicaterecord/workCommunicateRecord")
public class WorkCommunicateRecordController extends BaseController {

	@Autowired
	private WorkCommunicateRecordService workCommunicateRecordService;
	@Autowired
	private WorkCommunicateContentService workCommunicateContentService;
	
	@ModelAttribute
	public WorkCommunicateRecord get(@RequestParam(required=false) String id) {
		WorkCommunicateRecord entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workCommunicateRecordService.get(id);
		}
		if (entity == null){
			entity = new WorkCommunicateRecord();
		}
		return entity;
	}
	
	/**
	 * 沟通记录表列表页面
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateRecord:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkCommunicateRecord workCommunicateRecord, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkCommunicateRecord> page = workCommunicateRecordService.findPage(new Page<WorkCommunicateRecord>(request, response), workCommunicateRecord); 
		model.addAttribute("page", page);
		return "modules/workcommunicaterecord/workCommunicateRecordList";
	}

	/**
	 * 查看，增加，编辑沟通记录表表单页面
	 */
//	@RequiresPermissions(value={"workcommunicaterecord:workCommunicateRecord:view","workcommunicaterecord:workCommunicateRecord:add","workcommunicaterecord:workCommunicateRecord:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkCommunicateRecord workCommunicateRecord, HttpServletRequest request,Model model) {
		String view = request.getParameter("view");
		if("view".equals(view)){
			view = "modules/workcommunicaterecord/workCommunicateRecordFormView";
		}else{
			view = "modules/workcommunicaterecord/workCommunicateRecordForm";
		}
		if(!Strings.isNullOrEmpty(workCommunicateRecord.getProjectId())){
			workCommunicateRecord = workCommunicateRecordService.findListByProId(workCommunicateRecord);
			if(!workCommunicateRecord.getIsNewRecord()){
				WorkCommunicateContent workCommunicateContent = new WorkCommunicateContent();
				workCommunicateContent.setCommunicateId(workCommunicateRecord.getId());
				List<WorkCommunicateContent> workCommunicateContentList = workCommunicateContentService.findListByCommunicateId(workCommunicateContent);
				workCommunicateRecord.setWorkCommunicateContentList(workCommunicateContentList);
			}
		}

		model.addAttribute("workCommunicateRecord", workCommunicateRecord);
		return view;
	}

	/**
	 * 保存沟通记录表
	 */
//	@RequiresPermissions(value={"workcommunicaterecord:workCommunicateRecord:add","workcommunicaterecord:workCommunicateRecord:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkCommunicateRecord workCommunicateRecord, HttpServletRequest request,Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workCommunicateRecord)){
			return form(workCommunicateRecord, request,model);
		}
		if(!workCommunicateRecord.getIsNewRecord()){//编辑表单保存
			WorkCommunicateRecord t = workCommunicateRecordService.get(workCommunicateRecord.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workCommunicateRecord, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workCommunicateRecordService.save(t);//保存
		}else{//新增表单保存
			workCommunicateRecordService.save(workCommunicateRecord);//保存
		}
		addMessage(redirectAttributes, "保存沟通记录表成功");
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateRecord/?repage";
	}
	
	/**
	 * 删除沟通记录表
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateRecord:del")
	@RequestMapping(value = "delete")
	public String delete(WorkCommunicateRecord workCommunicateRecord, RedirectAttributes redirectAttributes) {
		workCommunicateRecordService.delete(workCommunicateRecord);
		addMessage(redirectAttributes, "删除沟通记录表成功");
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateRecord/?repage";
	}
	
	/**
	 * 批量删除沟通记录表
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateRecord:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workCommunicateRecordService.delete(workCommunicateRecordService.get(id));
		}
		addMessage(redirectAttributes, "删除沟通记录表成功");
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateRecord/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateRecord:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkCommunicateRecord workCommunicateRecord, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "沟通记录表"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkCommunicateRecord> page = workCommunicateRecordService.findPage(new Page<WorkCommunicateRecord>(request, response, -1), workCommunicateRecord);
    		new ExportExcel("沟通记录表", WorkCommunicateRecord.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出沟通记录表记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateRecord/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateRecord:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkCommunicateRecord> list = ei.getDataList(WorkCommunicateRecord.class);
			for (WorkCommunicateRecord workCommunicateRecord : list){
				try{
					workCommunicateRecordService.save(workCommunicateRecord);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条沟通记录表记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条沟通记录表记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入沟通记录表失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateRecord/?repage";
    }
	
	/**
	 * 下载导入沟通记录表数据模板
	 */
//	@RequiresPermissions("workcommunicaterecord:workCommunicateRecord:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "沟通记录表数据导入模板.xlsx";
    		List<WorkCommunicateRecord> list = Lists.newArrayList(); 
    		new ExportExcel("沟通记录表数据", WorkCommunicateRecord.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcommunicaterecord/workCommunicateRecord/?repage";
    }
	
	
	

}