/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.workcontent.entity.WorkContentBiddingplan;
import com.jeeplus.modules.workcontent.service.WorkContentBiddingplanService;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;

/**
 * 招标策划Controller
 * @author ssrh
 * @version 2018-06-05
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentBiddingplan")
public class WorkContentBiddingplanController extends BaseController {

	@Autowired
	private WorkContentBiddingplanService workContentBiddingplanService;
	
	@ModelAttribute
	public WorkContentBiddingplan get(@RequestParam(required=false) String id) {
		WorkContentBiddingplan entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentBiddingplanService.get(id);
		}
		if (entity == null){
			entity = new WorkContentBiddingplan();
		}
		return entity;
	}
	
	/**
	 * 招标策划列表页面
	 */
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentBiddingplan workContentBiddingplan, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentBiddingplan> page = workContentBiddingplanService.findPage(new Page<WorkContentBiddingplan>(request, response), workContentBiddingplan); 
		model.addAttribute("page", page);
		return "modules/workcontentbiddingplan/workContentBiddingplanList";
	}

	/**
	 * 查看，增加，编辑招标策划表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentBiddingplan workContentBiddingplan,String view, Model model) {
        WorkContentBiddingplan oldRecord = null;
        if(StringUtils.isNotBlank(workContentBiddingplan.getContentId())){
            List<WorkContentBiddingplan> list = workContentBiddingplanService.findList(workContentBiddingplan);
            if(list!=null&&list.size()>0){
                oldRecord = list.get(0);
            }
        }
        if(oldRecord!=null){
            model.addAttribute("workContentBiddingplan", oldRecord);
        }else {
            model.addAttribute("workContentBiddingplan", workContentBiddingplan);
        }

        String viewPath = "modules/workcontent/workContentBiddingplan";
        if("view".equals(view)){
            viewPath+="View";
        }
        return viewPath;
	}

	/**
	 * 保存招标策划
	 */
	@RequestMapping(value = "save")
	public String save(WorkContentBiddingplan workContentBiddingplan, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentBiddingplan)){
			return form(workContentBiddingplan, null,model);
		}
		if(!workContentBiddingplan.getIsNewRecord()){//编辑表单保存
			WorkContentBiddingplan t = workContentBiddingplanService.get(workContentBiddingplan.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentBiddingplan, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentBiddingplanService.save(t);//保存
		}else{//新增表单保存
			workContentBiddingplanService.save(workContentBiddingplan);//保存
		}
		addMessage(redirectAttributes, "保存招标策划成功");
		return "redirect:"+Global.getAdminPath()+"/workcontentbiddingplan/workContentBiddingplan/?repage";
	}
	
	/**
	 * 删除招标策划
	 */
	@RequestMapping(value = "delete")
	public String delete(WorkContentBiddingplan workContentBiddingplan, RedirectAttributes redirectAttributes) {
		workContentBiddingplanService.delete(workContentBiddingplan);
		addMessage(redirectAttributes, "删除招标策划成功");
		return "redirect:"+Global.getAdminPath()+"/workcontentbiddingplan/workContentBiddingplan/?repage";
	}
	
	/**
	 * 批量删除招标策划
	 */
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentBiddingplanService.delete(workContentBiddingplanService.get(id));
		}
		addMessage(redirectAttributes, "删除招标策划成功");
		return "redirect:"+Global.getAdminPath()+"/workcontentbiddingplan/workContentBiddingplan/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentBiddingplan workContentBiddingplan, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "招标策划"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentBiddingplan> page = workContentBiddingplanService.findPage(new Page<WorkContentBiddingplan>(request, response, -1), workContentBiddingplan);
    		new ExportExcel("招标策划", WorkContentBiddingplan.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出招标策划记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontentbiddingplan/workContentBiddingplan/?repage";
    }

	/**
	 * 导入Excel数据

	 */
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentBiddingplan> list = ei.getDataList(WorkContentBiddingplan.class);
			for (WorkContentBiddingplan workContentBiddingplan : list){
				try{
					workContentBiddingplanService.save(workContentBiddingplan);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条招标策划记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条招标策划记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入招标策划失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontentbiddingplan/workContentBiddingplan/?repage";
    }
	
	/**
	 * 下载导入招标策划数据模板
	 */
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "招标策划数据导入模板.xlsx";
    		List<WorkContentBiddingplan> list = Lists.newArrayList(); 
    		new ExportExcel("招标策划数据", WorkContentBiddingplan.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontentbiddingplan/workContentBiddingplan/?repage";
    }
	
	
	

}