/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcontent.entity.WorkContentContprogram;
import com.jeeplus.modules.workcontent.service.WorkContentContprogramService;

/**
 * 合约规划Controller
 * @author ssrh
 * @version 2018-06-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentContprogram")
public class WorkContentContprogramController extends BaseController {

	@Autowired
	private WorkContentContprogramService workContentContprogramService;
	
	@ModelAttribute
	public WorkContentContprogram get(@RequestParam(required=false) String id) {
		WorkContentContprogram entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentContprogramService.get(id);
		}
		if (entity == null){
			entity = new WorkContentContprogram();
		}
		return entity;
	}
	
	/**
	 * 合约规划列表页面
	 */
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentContprogram workContentContprogram, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentContprogram> page = workContentContprogramService.findPage(new Page<WorkContentContprogram>(request, response), workContentContprogram); 
		model.addAttribute("page", page);
		return "modules/workcontent/workContentContprogramList";
	}

	/**
	 * 查看，增加，编辑合约规划表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentContprogram workContentContprogram, String view,Model model) {
        WorkContentContprogram oldContprogram = null;
	    if(StringUtils.isNotBlank(workContentContprogram.getContentId())){
            List<WorkContentContprogram> contprograms = workContentContprogramService.findList(workContentContprogram);
            if(contprograms!=null&&contprograms.size()>0){
                oldContprogram = contprograms.get(0);
            }
        }
        if(oldContprogram!=null){
            model.addAttribute("workContentContprogram", oldContprogram);
        }else {
            model.addAttribute("workContentContprogram", workContentContprogram);
        }
        String viewPath = "modules/workcontent/workContentContprogram";
        if("view".equals(view)){
            viewPath+="View";
        }
        return viewPath;
	}

	/**
	 * 保存合约规划
	 */
	@RequestMapping(value = "save")
	public String save(WorkContentContprogram workContentContprogram, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentContprogram)){
			return form(workContentContprogram, null,model);
		}
		if(!workContentContprogram.getIsNewRecord()){//编辑表单保存
			WorkContentContprogram t = workContentContprogramService.get(workContentContprogram.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentContprogram, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentContprogramService.save(t);//保存
		}else{//新增表单保存
			workContentContprogramService.save(workContentContprogram);//保存
		}
		addMessage(redirectAttributes, "保存合约规划成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContprogram/?repage";
	}
	
	/**
	 * 删除合约规划
	 */
	@RequestMapping(value = "delete")
	public String delete(WorkContentContprogram workContentContprogram, RedirectAttributes redirectAttributes) {
		workContentContprogramService.delete(workContentContprogram);
		addMessage(redirectAttributes, "删除合约规划成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContprogram/?repage";
	}
	
	/**
	 * 批量删除合约规划
	 */
	@RequiresPermissions("workcontent:workContentContprogram:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentContprogramService.delete(workContentContprogramService.get(id));
		}
		addMessage(redirectAttributes, "删除合约规划成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContprogram/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcontent:workContentContprogram:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentContprogram workContentContprogram, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "合约规划"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentContprogram> page = workContentContprogramService.findPage(new Page<WorkContentContprogram>(request, response, -1), workContentContprogram);
    		new ExportExcel("合约规划", WorkContentContprogram.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出合约规划记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContprogram/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcontent:workContentContprogram:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentContprogram> list = ei.getDataList(WorkContentContprogram.class);
			for (WorkContentContprogram workContentContprogram : list){
				try{
					workContentContprogramService.save(workContentContprogram);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条合约规划记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条合约规划记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入合约规划失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContprogram/?repage";
    }
	
	/**
	 * 下载导入合约规划数据模板
	 */
	@RequiresPermissions("workcontent:workContentContprogram:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "合约规划数据导入模板.xlsx";
    		List<WorkContentContprogram> list = Lists.newArrayList(); 
    		new ExportExcel("合约规划数据", WorkContentContprogram.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentContprogram/?repage";
    }
	
	
	

}