/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workfullconstructsheet.service;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.jeeplus.common.oss.OSSClientUtil;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.service.CrudService;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MenuStatusEnum;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.modules.act.service.ActTaskService;
import com.jeeplus.modules.act.utils.ActUtils;
import com.jeeplus.modules.sys.entity.Role;
import com.jeeplus.modules.sys.entity.User;
import com.jeeplus.modules.sys.entity.Workattachment;
import com.jeeplus.modules.sys.service.OfficeService;
import com.jeeplus.modules.sys.service.WorkattachmentService;
import com.jeeplus.modules.sys.utils.UserUtils;
import com.jeeplus.modules.workactivity.entity.Activity;
import com.jeeplus.modules.workactivity.entity.WorkActivityProcess;
import com.jeeplus.modules.workactivity.service.ActivityService;
import com.jeeplus.modules.workactivity.service.WorkActivityProcessService;
import com.jeeplus.modules.workactivitymenu.entity.WorkActivityMenu;
import com.jeeplus.modules.workactivitymenu.service.WorkActivityMenuService;
import com.jeeplus.modules.workfullconstructsheet.dao.WorkFullConstructsheetDao;
import com.jeeplus.modules.workfullconstructsheet.entity.WorkFullConstructsheet;
import com.jeeplus.modules.workfullmanage.dao.WorkFullManageDao;
import com.jeeplus.modules.workfullmanage.entity.WorkFullManage;
import com.jeeplus.modules.workfullmanage.service.WorkFullManageService;
import com.jeeplus.modules.workprojectnotify.entity.WorkProjectNotify;
import com.jeeplus.modules.workprojectnotify.service.WorkProjectNotifyService;
import org.activiti.engine.ActivitiObjectNotFoundException;
import org.activiti.engine.IdentityService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.runtime.ProcessInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 施工联系单Service
 * @author fgy
 * @version 2017-11-06
 */
@SuppressWarnings("all")
@Service
@Transactional(readOnly = true)
public class WorkFullConstructsheetService extends CrudService<WorkFullConstructsheetDao, WorkFullConstructsheet> {

	@Autowired
	private WorkProjectNotifyService workProjectNotifyService;
	@Autowired
	private ActTaskService actTaskService;
	@Autowired
	private RuntimeService runtimeService;
	@Autowired
	private IdentityService identityService;
	@Autowired
	private WorkFullConstructsheetDao workFullConstructsheetDao;
	@Autowired
	private WorkFullManageService workFullManageService;
	@Autowired
	private WorkattachmentService workattachmentService;
	@Autowired
	private WorkFullManageDao workFullManageDao;
	@Autowired
	private WorkActivityMenuService workActivityMenuService;
	@Autowired
	private WorkActivityProcessService workActivityProcessService;
	@Autowired
	private ActivityService activityService;
	@Autowired
	private OfficeService officeService;

	private Lock lock = new ReentrantLock();

	public WorkFullConstructsheet get(String id) {

		WorkFullConstructsheet workFullConstructsheet = super.get(id);
		if(workFullConstructsheet != null){
			WorkFullManage workFullManage = workFullManageService.get(workFullConstructsheet.getWorkFullManageId());
			if(workFullManage != null){
				workFullConstructsheet.setWorkFullManage(workFullManage);
				workFullConstructsheet.setWorkFullManageId(workFullManage.getId());
			}

			List<Workattachment> attachmentList = workattachmentService.getListByAttachmentIdAndFlag(workFullConstructsheet.getId(),"47");
			if(attachmentList!=null && attachmentList.size()>0){
				for (int i = 0; i < attachmentList.size(); i++) {
					Workattachment attachment = attachmentList.get(i);
					if(attachment != null){
						attachment.setAttachmentUser(UserUtils.get(attachment.getAttachmentUser()).getName());
					}
				}
				workFullConstructsheet.setWorkAttachments(attachmentList);
			}
			//流程结束state=false，未结束state=true
			boolean state = true;
			if(StringUtils.isNotBlank(workFullConstructsheet.getProcessInstanceId())){
				state = actTaskService.isProcessEnd(workFullConstructsheet.getProcessInstanceId());
			}else{
				//如果是项目负责人，不用审核，没有流程实例
				state = false;
			}
			workFullConstructsheet.setProcessEnded(state);
		}

		return workFullConstructsheet ;
	}
	
	public List<WorkFullConstructsheet> findList(WorkFullConstructsheet workFullConstructsheet) {
		return super.findList(workFullConstructsheet);
	}
	
	public Page<WorkFullConstructsheet> findPage(Page<WorkFullConstructsheet> page, WorkFullConstructsheet workFullConstructsheet) {
		workFullConstructsheet.setPage(page);
		page.setList(findListBelongtoWorkfullmanage(workFullConstructsheet));
		return page;
	}

	private List<WorkFullConstructsheet> findListBelongtoWorkfullmanage(WorkFullConstructsheet workFullConstructsheet) {
		if(StringUtils.isBlank(workFullConstructsheet.getWorkFullManageId())){
			return null ;
		}
		List<WorkFullConstructsheet> constructsheetListList = workFullConstructsheetDao.findList(workFullConstructsheet);
		if(constructsheetListList != null && constructsheetListList.size()>0){
			for(WorkFullConstructsheet constructsheet : constructsheetListList){
				WorkFullManage workFullManage = workFullManageService.get(constructsheet.getWorkFullManageId());
				constructsheet.setWorkFullManage(workFullManage);
				constructsheet.setWorkFullManageId(workFullManage.getId());

				//流程结束state=false，未结束state=true
				boolean state = true;
				if(StringUtils.isNotBlank(constructsheet.getProcessInstanceId())){
					state = actTaskService.isProcessEnd(constructsheet.getProcessInstanceId());
				}else{
					//如果是项目负责人，不用审核，没有流程实例
					state = false;
				}
				constructsheet.setProcessEnded(state);

			}
		}
		return constructsheetListList;
	}

	@Transactional(readOnly = false)
	public WorkFullConstructsheet save(WorkFullConstructsheet workFullConstructsheet, String workFullManageId, String submit) {
		//(0暂存，1审核中，2已通过，3未通过、4已撤销)
		String id = workFullConstructsheet.getId();
		//提交、暂存、重新申请，执行保存，撤销不执行保存
		if("submit".equals(submit) || "toStore".equals(submit)|| "reapply".equals(submit)){
			if(StringUtils.isBlank(workFullConstructsheet.getId())){
				workFullConstructsheet.setStatus("0");
			}
			workFullConstructsheet.setWorkFullManageId(workFullManageId);
			workFullConstructsheet.setCompanyId(UserUtils.getSelectCompany().getId());
			workFullConstructsheet.setOfficeId(UserUtils.getSelectOffice().getId());
			super.save(workFullConstructsheet);
			//uploadFile(attachment_file,workFullConstructsheet,"47");
			if(StringUtils.isBlank(id)){
				List<Workattachment> workattachmentList = workFullConstructsheet.getWorkAttachments();
				if(workattachmentList!=null && workattachmentList.size()>0){
					for (int i = 0; i < workattachmentList.size(); i++) {
						Workattachment w = workattachmentList.get(i);
						if (StringUtils.isNotBlank(w.getUrl()) && !w.getUrl().equals("null")) {
							w.setCompanyId(workFullConstructsheet.getCompanyId());
							w.setAttachmentUser(UserUtils.getUser().getId());
							w.setAttachmentId(workFullConstructsheet.getId());
							w.setId("");
							workattachmentService.save(w);
						}
					}
				}
			}
		}

		//新添加数据：先保存，后启动流程；暂存数据：只保存，不启动流程；
		//重新申请：先保存数据，后执行审核；撤销：不保存，但是执行审核
		if("submit".equals(submit)){
			Map<String,Object> variables = new HashMap<String,Object>();
			startProcess(workFullConstructsheet,variables);
		}else if("reapply".equals(submit) || "repeal".equals(submit)){
			auditSave(workFullConstructsheet);
		}
		return workFullConstructsheet;
	}
	@Transactional(readOnly = false)
	private void startProcess(WorkFullConstructsheet workFullConstructsheet, Map<String, Object> variables) {
		System.err.print("==========审核流程开始=========");
		//用来设置启动流程的人员的id，引擎会自动把人员的id保存到activiti:initiator中
		identityService.setAuthenticatedUserId(workFullConstructsheet.getCurrentUser().getId());
		WorkActivityMenu workActivityMenu = workActivityMenuService.findByParentAndOffice("db2322f3c5db4648b2306b8089a8529b", officeService.get(workFullConstructsheet.getOfficeId()));
		//启动流程
		String businessKey = workFullConstructsheet.getId().toString();
		String processType = workActivityMenu.getProcessType();
		StringBuffer buffer1 = new StringBuffer();
		Activity activity = new Activity();
		if(StringUtils.isNotBlank(workActivityMenu.getId())){
			workActivityMenu = workActivityMenuService.get(workActivityMenu.getId());
			List<Activity> activities = workActivityMenu.getActivities();
			for (Activity a:activities){
				if (a.getDelFlag().equals("0") && a.getCount() == 1){
					activity = a;
					break;
				}
			}
			buffer1.append(activity.getRole().getEnname());
			variables.put("type", workActivityMenu.getProcessType());
		}else{
			processType= ActUtils.PD_WORKFULLCONSTRUCTSHEET[0];
			variables.put("type", processType);
			Role role = UserUtils.getSelectRole().get(0);
			buffer1 = DateUtils.getByEnnme(role.getEnname());
			buffer1.append("bmzr");
			variables.put("bmzr", buffer1.toString());
		}
		variables.put("businessId",businessKey);
		variables.put("title",workFullConstructsheet.getConstructsheetTopic());

		ProcessInstance processInstance = runtimeService.startProcessInstanceByKey(processType,businessKey,variables);
		workFullConstructsheet.setProcessInstance(processInstance);
		workFullConstructsheet.setProcessInstanceId(processInstance.getId());
		workFullConstructsheetDao.updateProcessInstanceId(workFullConstructsheet);

		workFullConstructsheet.setStatus("1");
		updateStatus(workFullConstructsheet);

		List<Activity> list= workActivityMenu.getActivities();
		if (list!=null && list.size()!=0){
			workActivityProcessService.saveList(list,processInstance.getId());
		}else {
			WorkActivityProcess workActivityProcess = new WorkActivityProcess();
			workActivityProcess.setProcessKey(processType);
			workActivityProcess.setCount(1);
			workActivityProcess.setProcessInstanceId(processInstance.getId());
			workActivityProcess.setIsApproval("0");
			workActivityProcess.setActivity(new Activity("86bba2f2095b43899311e8abdc623f44"));
			workActivityProcessService.save(workActivityProcess);
		}

		//发起通知给审批人
		WorkProjectNotify workProjectNotify = new WorkProjectNotify();
		workProjectNotify.setTitle("您有新的施工联系单待审批");
		workProjectNotify.setNotifyId(workFullConstructsheet.getId());
		workProjectNotify.setCompanyId(workFullConstructsheet.getCompanyId());
		workProjectNotify.setContent("审批任务-施工联系单");
		workProjectNotify.setRemarks("待审批");
		workProjectNotify.setType("10");
		workProjectNotify.setStatus("0");
		workProjectNotify.setNotifyRole("");
		workProjectNotify.setUser(new User(""));

		if(workActivityMenu!=null&&StringUtils.isNotBlank(workActivityMenu.getId())&&StringUtils.isNotBlank(activity.getId())){
			//角色审批
			if(StringUtils.isNotBlank(activity.getRole().getEnname())){
				workProjectNotify.setNotifyRole(buffer1.toString());
			}
			//人员审批
			if(StringUtils.isNotBlank(activity.getUser().getId())){
				workProjectNotify.setUser(new User(activity.getUser().getId()));
			}
		}else {
			workProjectNotify.setNotifyRole(buffer1.toString());
		}
		workProjectNotifyService.save(workProjectNotify);
	}
	@Transactional(readOnly = false)
	private void updateStatus(WorkFullConstructsheet workFullConstructsheet) {
		workFullConstructsheetDao.updateStatus(workFullConstructsheet);
	}

	@Transactional(readOnly = false)
	private void uploadFile(MultipartFile file, WorkFullConstructsheet workFullConstructsheet, String attachmentFlag) {
		if(file!=null && !file.isEmpty() && file.getSize()>0){
			//后一次上传前先删除前一次上传的文件
			delFile(workFullConstructsheet,attachmentFlag);

			OSSClientUtil ossClientUtil = new OSSClientUtil();
			String fileName = file.getOriginalFilename();
			String fileType = fileName.substring(fileName.lastIndexOf(".")+1);
			String url = ossClientUtil.uploadFile2OSS(file,"workFullConstructsheet");
			Workattachment workattachment = new Workattachment();
			workattachment.setUrl(url);
			workattachment.setType(fileType);
			workattachment.setAttachmentName(fileName);
			workattachment.setAttachmentUser(UserUtils.getUser().getId());
			workattachment.setAttachmentId(workFullConstructsheet.getId());
			workattachment.setAttachmentFlag(attachmentFlag);
			workattachmentService.save(workattachment);
		}

	}
	@Transactional(readOnly = false)
	public void delFile(WorkFullConstructsheet workFullConstructsheet, String attachmentFlag) {
		Workattachment workattachment = workattachmentService.get(workFullConstructsheet.getRemarks());
		if(workattachment != null){
			doDelFile(workattachment);
		}
	}
	@Transactional(readOnly = false)
	private void doDelFile(Workattachment workattachment) {
		workattachmentService.deleteFileFromAliyun(workattachment,workattachment==null?"":workattachment.getUrl());
		workattachmentService.delete(workattachment);

	}

	@Transactional(readOnly = false)
	public void delete(WorkFullConstructsheet workFullConstructsheet) {
		delFile(workFullConstructsheet,"47");
		super.delete(workFullConstructsheet);
		//逻辑删除相关的通知记录
		WorkProjectNotify notify = new WorkProjectNotify();
		notify.setNotifyId(workFullConstructsheet.getId());
		workProjectNotifyService.modifyDelflag(notify);
	}

	public Page<WorkFullManage> findListOfWorkFullManage(Page<WorkFullManage> page, WorkFullManage workFullManage) {
		if (StringUtils.isNotBlank(workFullManage.getTotalFullIds())){
			workFullManage.setTotalFullIds(","+workFullManage.getTotalFullIds()+",");
		}
		workFullManage.getSqlMap().put("dsf", dataScopeFilterOR(workFullManage.getCurrentUser(), "o", "u","s", MenuStatusEnum.END.getValue()));
		workFullManage.setPage(page);
		List<WorkFullManage> list = Lists.newArrayList();
		workFullManage.setIsTotal("1");
		List<WorkFullManage> totals = getListOfWorkFullManage(workFullManage);
		List<WorkFullManage> workFullManages = new ArrayList<>();
		for (WorkFullManage fullManage:totals){
			WorkFullManage wfm = new WorkFullManage();
			wfm.setTotalFullId(fullManage.getId());
			List<WorkFullManage> workEngineeringProjectLists = getListOfWorkFullManage(wfm);
			if (workEngineeringProjectLists!=null && workEngineeringProjectLists.size()!=0){
				workFullManages.addAll(workEngineeringProjectLists);
			}
		}
		workFullManages.addAll(totals);
		WorkFullManageService.sortList(list,workFullManages,"0",true);
		//WorkFullManageService.sortList(list,getListOfWorkFullManage(workFullManage),"0",true);
		page.setList(list);
		return page ;
	}

	private List<WorkFullManage> getListOfWorkFullManage(WorkFullManage workFullManage) {
		List<WorkFullManage> workFullManageList = workFullConstructsheetDao.getListOfWorkFullManage(workFullManage);
		if (workFullManageList != null && workFullManageList.size()>0){
			for(WorkFullManage w : workFullManageList){
				String hostMasterIds = w.getHostMaster();
				StringBuilder hostMasterName = new StringBuilder();
				String[] idArray = hostMasterIds.split(",");
				if(idArray != null && idArray.length>0){
					for(int i =0 ;i<idArray.length;i++){
						if(StringUtils.isNotBlank(idArray[i])){
							User u = UserUtils.get(idArray[i]);
							hostMasterName.append("，").append(u.getName());
						}
					}
				}
				if(hostMasterName.length()>1){
					w.setHostMasterName(hostMasterName.substring(1));
				}
			}
		}
		return workFullManageList ;
	}

	public WorkFullConstructsheet getByProcessInstanceId(String processInstanceId) {
		return workFullConstructsheetDao.getByProcessInstanceId(processInstanceId);
	}
	//部门负责人审核或创建人重申、撤销(0暂存，1审核中，2通过，3未通过，4已撤销)
	@Transactional(readOnly = false)
	public String auditSave(WorkFullConstructsheet workFullConstructsheet) {
		try {
			String userId = UserUtils.getUser().getId();
			actTaskService.claim(workFullConstructsheet.getAct().getTaskId(), userId);
			// 设置意见
			String comment = "";
			if(workFullConstructsheet.getStatus().equals("3")){//如果是被驳回状态
				comment = ("yes".equals(workFullConstructsheet.getAct().getFlag())?"[重新申请] ":"[已撤销] ");
			}else{
				comment = ("yes".equals(workFullConstructsheet.getAct().getFlag())?"[同意] ":"[驳回] ")+workFullConstructsheet.getAct().getComment();
			}
			workFullConstructsheet.getAct().setComment(comment);
			workFullConstructsheet.setStatus("1");

			System.err.println("comment======"+workFullConstructsheet.getAct().getComment());
			WorkProjectNotify workProjectNotify = new WorkProjectNotify();
			workProjectNotify.setNotifyId(workFullConstructsheet.getId());
			workProjectNotifyService.readByNotifyId(workProjectNotify);
			// 对不同环节的业务逻辑进行操作
			String taskDefKey = workFullConstructsheet.getAct().getTaskDefKey();
			System.err.println("taskDefKey======"+taskDefKey);
			//业务逻辑对应的条件表达式
			String exp = "";

			//WorkActivityMenu workActivityMenu = workActivityMenuService.findByParentAndOffice("db2322f3c5db4648b2306b8089a8529b",officeService.get(workFullConstructsheet.getOfficeId()));
			List<Activity> activitieList = activityService.getByProcessInstanceId(workFullConstructsheet.getProcessInstanceId());
			WorkActivityMenu workActivityMenu = new WorkActivityMenu();
			if (activitieList!=null && activitieList.size()!=0){
				workActivityMenu.setProcessType(activitieList.get(0).getProcessKey());
				workActivityMenu.setActivities(activitieList);
			}

			WorkActivityProcess workActivityProcess = new WorkActivityProcess();
			WorkActivityProcess selectedProcess = new WorkActivityProcess();
			selectedProcess.setProcessInstanceId(workFullConstructsheet.getProcessInstanceId());
			List<WorkActivityProcess> workActivityProcesses = workActivityProcessService.findList(selectedProcess);

			boolean isCustomProcess = StringUtils.isNotBlank(workActivityMenu.getProcessType()) && !workActivityMenu.getProcessType().equals(ActUtils.PD_WORKFULLCONSTRUCTSHEET[0]);

			String taskCount = "";
			int key = 0;

			List<Activity> activities = workActivityMenu.getActivities();
			if (isCustomProcess) {
				key = 1;
				for (int i =0;i<workActivityProcesses.size();i++){
					WorkActivityProcess activityProcess = workActivityProcesses.get(i);
					String count = taskDefKey.replace("task","").replace("audit","");

					if(taskDefKey.contains("audit") && (activityProcess.getCount()+"").contains(count)){
						taskCount = count ;
						exp = "pass";
						workActivityProcess.setId("");
						workActivityProcess.setCount(0);
						if (!"yes".equals(workFullConstructsheet.getAct().getFlag())) {
							workFullConstructsheet.setStatus("4");
							workActivityProcess.setIsApproval("2");
						}else {
							workFullConstructsheet.setStatus("1");
							workActivityProcess.setIsApproval("1");
							workProjectNotify.setTitle("施工联系单：重新申请");
							workProjectNotify.setRemarks("重新申请");
						}
					}
					else if ((activityProcess.getCount()+"").contains(count)){
						taskCount = count ;

						workActivityProcess = activityProcess;
						workActivityProcess.setRemarks(comment);

						if (!activityProcess.getIsApproval().equals("0")){
							workActivityProcess.setId("");
						}
						exp = "pass";
						if (!"yes".equals(workFullConstructsheet.getAct().getFlag())) {
							workFullConstructsheet.setStatus("3");
							workActivityProcess.setIsApproval("2");
							workProjectNotify.setTitle("您的施工联系单申请被驳回!");
							workProjectNotify.setNotifyRole("");
							workProjectNotify.setUser(workFullConstructsheet.getCreateBy());
							String returnBack = "-1";
							for (Activity activity:activities){
								if(activity.getCount()==Integer.parseInt(count)){
									returnBack = activity.getReturnBack();
									break;
								}
							}
							if (returnBack.equals("0")) {
								workActivityProcess.setId("");
							}
						}else {
							workActivityProcess.setIsApproval("1");
							workProjectNotify.setTitle("施工联系单：待审批");
						}
					}
				}
			}else {
				workActivityMenu.setId(workActivityProcesses.get(0).getId());
				workActivityMenu.setProcessType(ActUtils.PD_WORKFULLCONSTRUCTSHEET[0]);
				for (int i = 0; i < workActivityProcesses.size(); i++) {
					WorkActivityProcess activityProcess = workActivityProcesses.get(i);
					String count = activityProcess.getCount() + "";

					if(! activityProcess.getIsApproval().equals("0")){
						workActivityProcess.setId("");
					}
					taskCount = "1" ; // 默认流程只有一个审批环节

					// 部门负责人审批
					if ("audit".equals(taskDefKey)  && count.equals("1")){
						workActivityProcess.setCount(1); //默认流程里部门负责人是第一个环节
						exp = "pass";
						if("yes".equals(workFullConstructsheet.getAct().getFlag())){
							workActivityProcess.setIsApproval("1");
							workFullConstructsheet.setStatus("2");
							workProjectNotify.setTitle("您的施工联系单申请已通过");
							workProjectNotify.setRemarks("待通知");
							workProjectNotify.setNotifyRole("");
							workProjectNotify.setUser(workFullConstructsheet.getCreateBy());
						}else{
							workActivityProcess.setIsApproval("2");

							workFullConstructsheet.setStatus("3");//驳回
							workProjectNotify.setRemarks("重新申请");
							workProjectNotify.setTitle("施工联系单申请被驳回!");
							workProjectNotify.setNotifyRole("");
							workProjectNotify.setUser(workFullConstructsheet.getCreateBy());
						}
					}
					//重申|撤销
					else if("reapply".equals(taskDefKey) && count.equals("0")){
						workActivityProcess.setId("");
						workActivityProcess.setCount(0);
						exp="pass";
						if("yes".equals(workFullConstructsheet.getAct().getFlag())){//重新申请
							workActivityProcess.setIsApproval("1");
							workProjectNotify.setTitle("施工联系单重新申请!");
							Role role = UserUtils.getSelectRole().get(0);
							StringBuffer buffer = DateUtils.getByEnnme(role.getEnname());
							buffer.append("bmzr");
							workProjectNotify.setNotifyRole(buffer.toString());//给部门负责人再次发送通知
							workProjectNotify.setUser(new User(""));//如果是重新申请，就不给创建人发通知，只给部门负责人发通知
							workProjectNotify.setRemarks("待审批");
							workFullConstructsheet.setStatus("1");
						}else{
							workActivityProcess.setIsApproval("2");
							workFullConstructsheet.setStatus("4");//撤销
						}
						break;
					}
					// 未知环节
				}
			}

			//更新自定义的流程追踪数据
			String defaultReapply = "reapply"; //默认流程中的重新申请任务标识
			workActivityProcessService.updateProcess(workActivityProcess,workActivityMenu,key,taskCount,workFullConstructsheet.getProcessInstanceId(),taskDefKey,defaultReapply,workFullConstructsheet.getAct().getFlag(),comment, activities);


			// 设置意见
			workFullConstructsheet.getAct().setComment(comment);
			workFullConstructsheet.preUpdate();

			// 提交流程任务
			Map<String, Object> vars = Maps.newHashMap();
			vars.put(exp, "yes".equals(workFullConstructsheet.getAct().getFlag()) ?true : false );
			System.err.println("vars======"+vars.toString());
			// 提交流程任务
			actTaskService.complete(workFullConstructsheet.getAct().getTaskId(), workFullConstructsheet.getAct().getProcInsId(), workFullConstructsheet.getAct().getComment(), vars);
			boolean state = actTaskService.isProcessEnd(workFullConstructsheet.getAct().getProcInsId());
			if (!state){
				if(isCustomProcess){
					if("yes".equals(workFullConstructsheet.getAct().getFlag())){
						workFullConstructsheet.setStatus("2");
					}else{
						if(! "3".equals(workFullConstructsheet.getStatus()))
							workFullConstructsheet.setStatus("4");
					}
				}

				//终审之后，发送通知给创建人
				if("2".equals(workFullConstructsheet.getStatus())){
					workProjectNotify.setTitle("您的施工联系单申请已通过");
				}
				if("3".equals(workFullConstructsheet.getStatus())){
					workProjectNotify.setTitle("您的施工联系单申请被驳回");
				}
				workProjectNotify.setRemarks("待通知");//用于在通知页面做查看操作，这一步必须
				workProjectNotify.setNotifyRole("");
				workProjectNotify.setUser(workFullConstructsheet.getCreateBy());
			}else{
				if (isCustomProcess) {
					String count = taskDefKey.replace("audit","").replace("task","");
					Activity activity = new Activity();

					if (taskDefKey.contains("audit")) {
						if (activities.size() == 1 || count.equals("1")){
							activity = activities.get(0);
						}else if (activities.size()==Integer.parseInt(count)){
							activity = activities.get(activities.size()-1);
							String returnBack = activity.getReturnBack();
							if (returnBack.equals("1")){
								activity = activities.get(Integer.parseInt(count)-2);
							}else if (returnBack.equals("2")){
								activity = activities.get(0);
							}
						}else {
							activity = activities.get(Integer.parseInt(count)-1);
							String returnBack = activity.getReturnBack();
							if (returnBack.equals("1")){
								activity = activities.get(Integer.parseInt(count)-2);
							}else if (returnBack.equals("2")){
								activity = activities.get(0);
							}
						}
					}

					else if(!"yes".equals(workFullConstructsheet.getAct().getFlag())){

						workProjectNotify.setRemarks("重新申请");
						activity = new Activity();
					}

					else {

						activity = activities.get(Integer.parseInt(count));
						workProjectNotify.setRemarks("待审批");
					}

					if(StringUtils.isNotBlank(activity.getId())){
						workProjectNotify.setUser(activity.getUser());
						workProjectNotify.setNotifyRole(activity.getRole().getEnname());
					}
				}
				else{

				}
			}

			workProjectNotify.setCompanyId(workFullConstructsheet.getCompanyId());
			workProjectNotify.setContent(workFullConstructsheet.getAct().getComment());
			workProjectNotify.setType("10");
			workProjectNotify.setStatus("0");
			//创建人如果做了撤销操作，无需发送通知
			if (!workFullConstructsheet.getStatus().equals("4")) {
				workProjectNotifyService.save(workProjectNotify);
			}
			//更新施工联系单审核状态
			workFullConstructsheetDao.updateStatus(workFullConstructsheet);

			System.err.println("施工联系单审核正常");
			return "审核执行成功";
			//如果是撤销操作，更改与此相关的所有通知的已读状态，不再显示
			//如果是同意或驳回操作：将前面发给审批人的所有通知变更为已读；
			//如果是重新申请操作：将前面发给申请人的所有通知变更为已读；
			//如果是同意操作，发送审批通过的通知给申请人；申请人在阅读完成后，变更已读状态；
//			WorkProjectNotify notify = new WorkProjectNotify();
//			notify.setNotifyId(workFullConstructsheet.getId());
//			if("4".equals(workFullConstructsheet.getStatus())){
//				workProjectNotifyService.readByNotifyId(notify);
//			}
//			else if("2".equals(workFullConstructsheet.getStatus()) ||"3".equals(workFullConstructsheet.getStatus())){
//				String roleEnname = DateUtils.getByEnnme(UserUtils.getSelectRole().getEnname())+"bmzr";
//				notify.setNotifyRole(roleEnname);
//				workProjectNotifyService.readByNotifyIdAndNotifyRole(notify);
//			}else if("1".equals(workFullConstructsheet.getStatus())){
//				notify.setUser(workFullConstructsheet.getCreateBy());
//				workProjectNotifyService.readByNotifyIdAndNotifyUser(notify);
//			}

		}catch (ActivitiObjectNotFoundException e){
			e.printStackTrace();
			System.err.println("施工联系单--重复审批");
			logger.error("ActivitiObjectNotFoundException -- "+e);
			return "流程已审批，不能重新审批！";
		}
		catch (Exception e) {
			e.printStackTrace();
			System.err.println("施工联系单审核异常");
			logger.error("Exception e:"+e);
			return "审核执行失败！！";
		}
	}


}