/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workieavetemplate.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workieavetemplate.entity.WorkIeaveTemplateUser;
import com.jeeplus.modules.workieavetemplate.service.WorkIeaveTemplateUserService;

/**
 * 离职模板Controller
 * @author chenyuping
 * @version 2018-08-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workieavetemplate/workIeaveTemplateUser")
public class WorkIeaveTemplateUserController extends BaseController {

	@Autowired
	private WorkIeaveTemplateUserService workIeaveTemplateUserService;
	
	@ModelAttribute
	public WorkIeaveTemplateUser get(@RequestParam(required=false) String id) {
		WorkIeaveTemplateUser entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workIeaveTemplateUserService.get(id);
		}
		if (entity == null){
			entity = new WorkIeaveTemplateUser();
		}
		return entity;
	}
	
	/**
	 * 离职模板列表页面
	 */
	@RequiresPermissions("workieavetemplate:workIeaveTemplateUser:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkIeaveTemplateUser workIeaveTemplateUser, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkIeaveTemplateUser> page = workIeaveTemplateUserService.findPage(new Page<WorkIeaveTemplateUser>(request, response), workIeaveTemplateUser); 
		model.addAttribute("page", page);
		return "modules/workieavetemplate/workIeaveTemplateUserList";
	}

	/**
	 * 查看，增加，编辑离职模板表单页面
	 */
	@RequiresPermissions(value={"workieavetemplate:workIeaveTemplateUser:view","workieavetemplate:workIeaveTemplateUser:add","workieavetemplate:workIeaveTemplateUser:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkIeaveTemplateUser workIeaveTemplateUser, Model model) {
		model.addAttribute("workIeaveTemplateUser", workIeaveTemplateUser);
		return "modules/workieavetemplate/workIeaveTemplateUserForm";
	}

	/**
	 * 保存离职模板
	 */
	@RequiresPermissions(value={"workieavetemplate:workIeaveTemplateUser:add","workieavetemplate:workIeaveTemplateUser:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkIeaveTemplateUser workIeaveTemplateUser, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workIeaveTemplateUser)){
			return form(workIeaveTemplateUser, model);
		}
		if(!workIeaveTemplateUser.getIsNewRecord()){//编辑表单保存
			WorkIeaveTemplateUser t = workIeaveTemplateUserService.get(workIeaveTemplateUser.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workIeaveTemplateUser, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workIeaveTemplateUserService.save(t);//保存
		}else{//新增表单保存
			workIeaveTemplateUserService.save(workIeaveTemplateUser);//保存
		}
		addMessage(redirectAttributes, "保存离职模板成功");
		return "redirect:"+Global.getAdminPath()+"/workieavetemplate/workIeaveTemplateUser/?repage";
	}
	
	/**
	 * 删除离职模板
	 */
	@RequiresPermissions("workieavetemplate:workIeaveTemplateUser:del")
	@RequestMapping(value = "delete")
	public String delete(WorkIeaveTemplateUser workIeaveTemplateUser, RedirectAttributes redirectAttributes) {
		workIeaveTemplateUserService.delete(workIeaveTemplateUser);
		addMessage(redirectAttributes, "删除离职模板成功");
		return "redirect:"+Global.getAdminPath()+"/workieavetemplate/workIeaveTemplateUser/?repage";
	}
	
	/**
	 * 批量删除离职模板
	 */
	@RequiresPermissions("workieavetemplate:workIeaveTemplateUser:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workIeaveTemplateUserService.delete(workIeaveTemplateUserService.get(id));
		}
		addMessage(redirectAttributes, "删除离职模板成功");
		return "redirect:"+Global.getAdminPath()+"/workieavetemplate/workIeaveTemplateUser/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workieavetemplate:workIeaveTemplateUser:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkIeaveTemplateUser workIeaveTemplateUser, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "离职模板"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkIeaveTemplateUser> page = workIeaveTemplateUserService.findPage(new Page<WorkIeaveTemplateUser>(request, response, -1), workIeaveTemplateUser);
    		new ExportExcel("离职模板", WorkIeaveTemplateUser.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出离职模板记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workieavetemplate/workIeaveTemplateUser/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workieavetemplate:workIeaveTemplateUser:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkIeaveTemplateUser> list = ei.getDataList(WorkIeaveTemplateUser.class);
			for (WorkIeaveTemplateUser workIeaveTemplateUser : list){
				try{
					workIeaveTemplateUserService.save(workIeaveTemplateUser);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条离职模板记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条离职模板记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入离职模板失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workieavetemplate/workIeaveTemplateUser/?repage";
    }
	
	/**
	 * 下载导入离职模板数据模板
	 */
	@RequiresPermissions("workieavetemplate:workIeaveTemplateUser:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "离职模板数据导入模板.xlsx";
    		List<WorkIeaveTemplateUser> list = Lists.newArrayList(); 
    		new ExportExcel("离职模板数据", WorkIeaveTemplateUser.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workieavetemplate/workIeaveTemplateUser/?repage";
    }
	
	
	

}