/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workitemcollar.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workitemcollar.entity.WorkItemCollarCatalog;
import com.jeeplus.modules.workitemcollar.service.WorkItemCollarCatalogService;

/**
 * 领用物品子表Controller
 * @author chenyuping
 * @version 2018-07-16
 */
@Controller
@RequestMapping(value = "${adminPath}/workitemcollar/workItemCollarCatalog")
public class WorkItemCollarCatalogController extends BaseController {

	@Autowired
	private WorkItemCollarCatalogService workItemCollarCatalogService;
	
	@ModelAttribute
	public WorkItemCollarCatalog get(@RequestParam(required=false) String id) {
		WorkItemCollarCatalog entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workItemCollarCatalogService.get(id);
		}
		if (entity == null){
			entity = new WorkItemCollarCatalog();
		}
		return entity;
	}
	
	/**
	 * 物品领用列表页面
	 */
	@RequiresPermissions("workitemcollar:workItemCollarCatalog:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkItemCollarCatalog workItemCollarCatalog, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkItemCollarCatalog> page = workItemCollarCatalogService.findPage(new Page<WorkItemCollarCatalog>(request, response), workItemCollarCatalog); 
		model.addAttribute("page", page);
		return "modules/workitemcollar/workItemCollarCatalogList";
	}

	/**
	 * 查看，增加，编辑物品领用表单页面
	 */
	@RequiresPermissions(value={"workitemcollar:workItemCollarCatalog:view","workitemcollar:workItemCollarCatalog:add","workitemcollar:workItemCollarCatalog:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkItemCollarCatalog workItemCollarCatalog, Model model) {
		model.addAttribute("workItemCollarCatalog", workItemCollarCatalog);
		return "modules/workitemcollar/workItemCollarCatalogForm";
	}

	/**
	 * 保存物品领用
	 */
	@RequiresPermissions(value={"workitemcollar:workItemCollarCatalog:add","workitemcollar:workItemCollarCatalog:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkItemCollarCatalog workItemCollarCatalog, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workItemCollarCatalog)){
			return form(workItemCollarCatalog, model);
		}
		if(!workItemCollarCatalog.getIsNewRecord()){//编辑表单保存
			WorkItemCollarCatalog t = workItemCollarCatalogService.get(workItemCollarCatalog.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workItemCollarCatalog, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workItemCollarCatalogService.save(t);//保存
		}else{//新增表单保存
			workItemCollarCatalogService.save(workItemCollarCatalog);//保存
		}
		addMessage(redirectAttributes, "保存物品领用成功");
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarCatalog/?repage";
	}
	
	/**
	 * 删除物品领用
	 */
	@RequiresPermissions("workitemcollar:workItemCollarCatalog:del")
	@RequestMapping(value = "delete")
	public String delete(WorkItemCollarCatalog workItemCollarCatalog, RedirectAttributes redirectAttributes) {
		workItemCollarCatalogService.delete(workItemCollarCatalog);
		addMessage(redirectAttributes, "删除物品领用成功");
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarCatalog/?repage";
	}
	
	/**
	 * 批量删除物品领用
	 */
	@RequiresPermissions("workitemcollar:workItemCollarCatalog:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workItemCollarCatalogService.delete(workItemCollarCatalogService.get(id));
		}
		addMessage(redirectAttributes, "删除物品领用成功");
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarCatalog/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workitemcollar:workItemCollarCatalog:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkItemCollarCatalog workItemCollarCatalog, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "物品领用"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkItemCollarCatalog> page = workItemCollarCatalogService.findPage(new Page<WorkItemCollarCatalog>(request, response, -1), workItemCollarCatalog);
    		new ExportExcel("物品领用", WorkItemCollarCatalog.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出物品领用记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarCatalog/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workitemcollar:workItemCollarCatalog:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkItemCollarCatalog> list = ei.getDataList(WorkItemCollarCatalog.class);
			for (WorkItemCollarCatalog workItemCollarCatalog : list){
				try{
					workItemCollarCatalogService.save(workItemCollarCatalog);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条物品领用记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条物品领用记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入物品领用失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarCatalog/?repage";
    }
	
	/**
	 * 下载导入物品领用数据模板
	 */
	@RequiresPermissions("workitemcollar:workItemCollarCatalog:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "物品领用数据导入模板.xlsx";
    		List<WorkItemCollarCatalog> list = Lists.newArrayList(); 
    		new ExportExcel("物品领用数据", WorkItemCollarCatalog.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarCatalog/?repage";
    }
	
	
	

}