/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workitemcollar.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workitemcollar.entity.WorkItemCollarItem;
import com.jeeplus.modules.workitemcollar.service.WorkItemCollarItemService;

/**
 * 领用固定资产子表Controller
 * @author chenyuping
 * @version 2018-07-16
 */
@Controller
@RequestMapping(value = "${adminPath}/workitemcollar/workItemCollarItem")
public class WorkItemCollarItemController extends BaseController {

	@Autowired
	private WorkItemCollarItemService workItemCollarItemService;
	
	@ModelAttribute
	public WorkItemCollarItem get(@RequestParam(required=false) String id) {
		WorkItemCollarItem entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workItemCollarItemService.get(id);
		}
		if (entity == null){
			entity = new WorkItemCollarItem();
		}
		return entity;
	}
	
	/**
	 * 领用固定资产列表页面
	 */
	@RequiresPermissions("workitemcollar:workItemCollarItem:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkItemCollarItem workItemCollarItem, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkItemCollarItem> page = workItemCollarItemService.findPage(new Page<WorkItemCollarItem>(request, response), workItemCollarItem); 
		model.addAttribute("page", page);
		return "modules/workitemcollar/workItemCollarItemList";
	}

	/**
	 * 查看，增加，编辑领用固定资产表单页面
	 */
	@RequiresPermissions(value={"workitemcollar:workItemCollarItem:view","workitemcollar:workItemCollarItem:add","workitemcollar:workItemCollarItem:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkItemCollarItem workItemCollarItem, Model model) {
		model.addAttribute("workItemCollarItem", workItemCollarItem);
		return "modules/workitemcollar/workItemCollarItemForm";
	}

	/**
	 * 保存领用固定资产
	 */
	@RequiresPermissions(value={"workitemcollar:workItemCollarItem:add","workitemcollar:workItemCollarItem:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkItemCollarItem workItemCollarItem, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workItemCollarItem)){
			return form(workItemCollarItem, model);
		}
		if(!workItemCollarItem.getIsNewRecord()){//编辑表单保存
			WorkItemCollarItem t = workItemCollarItemService.get(workItemCollarItem.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workItemCollarItem, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workItemCollarItemService.save(t);//保存
		}else{//新增表单保存
			workItemCollarItemService.save(workItemCollarItem);//保存
		}
		addMessage(redirectAttributes, "保存领用固定资产成功");
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarItem/?repage";
	}
	
	/**
	 * 删除领用固定资产
	 */
	@RequiresPermissions("workitemcollar:workItemCollarItem:del")
	@RequestMapping(value = "delete")
	public String delete(WorkItemCollarItem workItemCollarItem, RedirectAttributes redirectAttributes) {
		workItemCollarItemService.delete(workItemCollarItem);
		addMessage(redirectAttributes, "删除领用固定资产成功");
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarItem/?repage";
	}
	
	/**
	 * 批量删除领用固定资产
	 */
	@RequiresPermissions("workitemcollar:workItemCollarItem:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workItemCollarItemService.delete(workItemCollarItemService.get(id));
		}
		addMessage(redirectAttributes, "删除领用固定资产成功");
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarItem/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workitemcollar:workItemCollarItem:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkItemCollarItem workItemCollarItem, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "领用固定资产"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkItemCollarItem> page = workItemCollarItemService.findPage(new Page<WorkItemCollarItem>(request, response, -1), workItemCollarItem);
    		new ExportExcel("领用固定资产", WorkItemCollarItem.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出领用固定资产记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarItem/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workitemcollar:workItemCollarItem:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkItemCollarItem> list = ei.getDataList(WorkItemCollarItem.class);
			for (WorkItemCollarItem workItemCollarItem : list){
				try{
					workItemCollarItemService.save(workItemCollarItem);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条领用固定资产记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条领用固定资产记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入领用固定资产失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarItem/?repage";
    }
	
	/**
	 * 下载导入领用固定资产数据模板
	 */
	@RequiresPermissions("workitemcollar:workItemCollarItem:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "领用固定资产数据导入模板.xlsx";
    		List<WorkItemCollarItem> list = Lists.newArrayList(); 
    		new ExportExcel("领用固定资产数据", WorkItemCollarItem.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitemcollar/workItemCollarItem/?repage";
    }
	
	
	

}